import React, { useCallback, useEffect, useRef, useState } from 'react'
import dayjs from 'dayjs'
import { Link, useParams } from 'react-router-dom'
import { getNavigationsValue } from '@ijl/cli'
import { useForm, Controller } from 'react-hook-form'
import {
  Breadcrumb,
  BreadcrumbItem,
  BreadcrumbLink,
  Container,
  Box,
  Card,
  CardBody,
  CardHeader,
  Heading,
  Button,
  CloseButton,
  useToast,
  VStack,
  FormControl,
  FormLabel,
  Toast,
  FormHelperText,
  FormErrorMessage,
  Input,
  TableContainer,
  Table,
  Thead,
  Tr,
  Th,
  Tbody,
  Td,
  Menu,
  MenuButton,
  MenuItem,
  Text,
  MenuList,
  Center,
  Spinner,
  AlertDialog,
  AlertDialogBody,
  AlertDialogContent,
  AlertDialogFooter,
  AlertDialogHeader,
  AlertDialogOverlay,
} from '@chakra-ui/react'
import { AddIcon, EditIcon } from '@chakra-ui/icons'

import { ErrorSpan, BreadcrumbsWrapper } from './style'

import { useAppSelector } from '../__data__/store'
import { api } from '../__data__/api/api'
import { isTeacher } from '../utils/user'
import { qrCode } from '../assets'
import { Lesson } from '../__data__/model'

interface NewLessonForm {
  name: string
  date: string
}

const LessonList = () => {
  const { courseId } = useParams()
  const user = useAppSelector((s) => s.user)
  const { data, isLoading, error } = api.useLessonListQuery(courseId)
  const [createLesson, crLQuery] = api.useCreateLessonMutation()
  const [deleteLesson, deletingRqst] = api.useDeleteLessonMutation()
  const [showForm, setShowForm] = useState(false)
  const [lessonToDelete, setlessonToDelete] = useState<Lesson>(null)
  const cancelRef = React.useRef()
  const {
    control,
    handleSubmit,
    reset,
    formState: { errors },
    getValues,
  } = useForm<NewLessonForm>({
    defaultValues: {
      name: '',
      date: '',
    },
  })
  const toast = useToast()
  const toastRef = useRef(null)

  const onSubmit = ({ name, date }) => {
    toastRef.current = toast({
      title: 'Отправляем',
      status: 'loading',
      duration: 9000,
    })
    createLesson({ name, courseId, date })
  }

  useEffect(() => {
    if (deletingRqst.isError) {
      toast({
        title: (deletingRqst.error as any)?.error,
        status: 'error',
        duration: 3000,
      })
    }

    if (deletingRqst.isSuccess) {
      const lesson = { ...lessonToDelete }
      toast({
        status: 'warning',
        duration: 9000,
        render(props) {
          return (
            <Toast
              {...props}
              title={
                <>
                  <Box pb={3}>
                    <Text fontSize="xl">{`Удалена лекция ${lesson.name}`}</Text>
                  </Box>
                  <Button
                    onClick={() =>
                      createLesson({ courseId, ...lesson })
                    }
                  >
                    Восстановить
                  </Button>
                </>
              }
            />
          )
        },
      })
      setlessonToDelete(null)
    }
  }, [deletingRqst.isLoading, deletingRqst.isSuccess, deletingRqst.isError])

  useEffect(() => {
    if (crLQuery.isSuccess) {
      const values = getValues()
      if (toastRef.current) {
        toast.update(toastRef.current, {
          title: 'Лекция создана',
          description: `Лекция ${values.name} успешно создана`,
          status: 'success',
          duration: 9000,
          isClosable: true,
        })
      }
      reset()
    }
  }, [crLQuery.isSuccess])

  if (isLoading) {
    return (
      <Container maxW="container.xl">
        <Center h="300px">
          <Spinner
            thickness="4px"
            speed="0.65s"
            emptyColor="gray.200"
            color="blue.500"
            size="xl"
          />
        </Center>
      </Container>
    )
  }

  return (
    <>
      <AlertDialog
        isOpen={Boolean(lessonToDelete)}
        leastDestructiveRef={cancelRef}
        onClose={() => setlessonToDelete(null)}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              Удалить занятие от{' '}
              {dayjs(lessonToDelete?.date).format('DD.MM.YY')}?
            </AlertDialogHeader>

            <AlertDialogBody>
              Все данные о посещении данного занятия будут удалены
            </AlertDialogBody>

            <AlertDialogFooter>
              <Button
                isDisabled={deletingRqst.isLoading}
                ref={cancelRef}
                onClick={() => setlessonToDelete(null)}
              >
                Cancel
              </Button>
              <Button
                colorScheme="red"
                loadingText=""
                isLoading={deletingRqst.isLoading}
                onClick={() => deleteLesson(lessonToDelete._id)}
                ml={3}
              >
                Delete
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
      <BreadcrumbsWrapper>
        <Breadcrumb>
          <BreadcrumbItem>
            <BreadcrumbLink as={Link} to={getNavigationsValue('journal.main')}>
              Журнал
            </BreadcrumbLink>
          </BreadcrumbItem>

          <BreadcrumbItem isCurrentPage>
            <BreadcrumbLink href="#">Курс</BreadcrumbLink>
          </BreadcrumbItem>
        </Breadcrumb>
      </BreadcrumbsWrapper>
      <Container maxW="container.xl">
        {isTeacher(user) && (
          <Box mt="15" mb="15">
            {showForm ? (
              <Card align="left">
                <CardHeader display="flex">
                  <Heading as="h2" mt="0">
                    Создание лекции
                  </Heading>
                  <CloseButton ml="auto" onClick={() => setShowForm(false)} />
                </CardHeader>
                <CardBody>
                  <form onSubmit={handleSubmit(onSubmit)}>
                    <VStack spacing="10" align="left">
                      <Controller
                        control={control}
                        name="date"
                        rules={{ required: 'Обязательное поле' }}
                        render={({ field }) => (
                          <FormControl>
                            <FormLabel>Дата</FormLabel>
                            <Input
                              {...field}
                              required={false}
                              placeholder="Укажите дату лекции"
                              size="md"
                              type="datetime-local"
                            />
                            {errors.date ? (
                              <FormErrorMessage>
                                {errors.date?.message}
                              </FormErrorMessage>
                            ) : (
                              <FormHelperText>
                                Укажите дату и время лекции
                              </FormHelperText>
                            )}
                          </FormControl>
                        )}
                      />

                      <Controller
                        control={control}
                        name="name"
                        rules={{ required: 'Обязательное поле' }}
                        render={({ field }) => (
                          <FormControl
                            isRequired
                            isInvalid={Boolean(errors.name)}
                          >
                            <FormLabel>Название новой лекции:</FormLabel>
                            <Input
                              {...field}
                              required={false}
                              placeholder="Название лекции"
                              size="md"
                            />
                            {errors.name && (
                              <FormErrorMessage>
                                {errors.name.message}
                              </FormErrorMessage>
                            )}
                          </FormControl>
                        )}
                      />
                      <Box mt="10">
                        <Button
                          size="lg"
                          type="submit"
                          leftIcon={<AddIcon />}
                          colorScheme="blue"
                        >
                          Создать
                        </Button>
                      </Box>
                    </VStack>

                    {crLQuery.error && (
                      <ErrorSpan>{(crLQuery.error as any).error}</ErrorSpan>
                    )}
                  </form>
                </CardBody>
              </Card>
            ) : (
              <Box p="2" m="2">
                <Button
                  leftIcon={<AddIcon />}
                  colorScheme="green"
                  onClick={() => setShowForm(true)}
                >
                  Добавить
                </Button>
              </Box>
            )}
          </Box>
        )}
        <TableContainer whiteSpace="wrap">
          <Table variant="striped" colorScheme="cyan">
            <Thead>
              <Tr>
                {isTeacher(user) && (
                  <Th align="center" width={1}>
                    ссылка
                  </Th>
                )}
                <Th textAlign="center" width={1}>
                  Дата
                </Th>
                <Th>Название</Th>
                <Th>action</Th>
                <Th isNumeric>Отмечено</Th>
              </Tr>
            </Thead>
            <Tbody>
              {data?.body?.map((lesson) => (
                <Tr key={lesson._id}>
                  {isTeacher(user) && (
                    <Td>
                      <Link
                        to={`${getNavigationsValue('journal.main')}/lesson/${courseId}/${lesson._id}`}
                        style={{ display: 'flex' }}
                      >
                        <img
                          width={24}
                          src={qrCode}
                          style={{ margin: '0 auto' }}
                        />
                      </Link>
                    </Td>
                  )}
                  <Td textAlign="center">
                    {dayjs(lesson.date).format('H:mm DD.MM.YY')}
                  </Td>
                  <Td>{lesson.name}</Td>
                  <Td>
                    <Menu>
                      <MenuButton as={Button}>
                        <EditIcon />
                      </MenuButton>
                      <MenuList>
                        <MenuItem isDisabled>Edit</MenuItem>
                        <MenuItem onClick={() => setlessonToDelete(lesson)}>
                          Delete
                        </MenuItem>
                      </MenuList>
                    </Menu>
                  </Td>
                  <Td isNumeric>{lesson.students.length}</Td>
                </Tr>
              ))}
            </Tbody>
          </Table>
        </TableContainer>
      </Container>
    </>
  )
}

export default LessonList
